/*************************************************************************************************************************
									File Details
**************************************************************************************************************************
	File name 							: SPIInit.C
	
	Purpose								: This file has SPI port related drivers required for communicating with the FLASH
	
	Author								: Mitesh Moonat & Harshit Gaharwar
	
	VDSP version used for testing		: VisualDSP++ 5.0, Update 10
	
	Hardware used						: ADSP-21469, ADSP-21479, ADSP-21489, ADSP-21375, ADSP-21369, ADSP-21364,
										  ADSP-21262  EZ-Kits
	
	Connection details 					: See "main.C"
	
	Guidelines for change				: No change required for different SHARC processor or different FLASH device.
**************************************************************************************************************************/

/*************************************************************************************************************************
									Include Files
*************************************************************************************************************************/
#include "SPIInit.h"


/*************************************************************************************************************************
									Function Definitions
**************************************************************************************************************************/

/**************************************************************************************************************************
	Function name	:   SPI_Send_Receive_32bit;
    Description		:   This function sends and receives one 32 bit word over SPI in full duplex mode
	Arguments		:	Parameter			 |	Description		 							| Valid values
						uiTxBuff			 |  tx buffer value		 						| unsigned int 32 bit
						bMSBF				 |  send/receive the words in LSBF/MSBF format	| =true for MSBF = false for LSBF
	Return value	:   received 32 bit data
**************************************************************************************************************************/
unsigned int SPI_Send_Receive_32bit(unsigned int uiTxBuff, bool bMSBF)
{
	//Local variables
	unsigned int uiTemp=0;
	
	//Flush the DMA FIFO
	*pSPIDMAC=0x00000080;
	
	//Flush the SPI FIFO
	*pSPICTL=TXFLSH|RXFLSH;
	
	//Initialize DMA parameters and SPICTL to receive one 32 bit word and send the contents of TXSPI simulataneously
	*pIISPI=(unsigned int)&uiTemp;
	*pIMSPI=1;
	*pCSPI=1;
	
	if(bMSBF==true)
		*pSPICTL=SPIEN|SPIMS|WL32|SENDZ|CLKPL|CPHASE|TIMOD2|MSBF;
	else
		*pSPICTL=SPIEN|SPIMS|WL32|SENDZ|CLKPL|CPHASE|TIMOD2;
		
	*pTXSPI=uiTxBuff;
	*pSPIDMAC=0x00000003;
	
	//Wait till DMA is active
	while((*pSPIDMAC&SPIDMAS)!=0);	
	
	return uiTemp;
}


/**************************************************************************************************************************
	Function name	:   SPI_Send_Receive_16bit;
    Description		:   This function sends and receives one 16 bit word over SPI in full duplex mode
	Arguments		:	Parameter			 |	Description		 							| Valid values
						uiTxBuff			 |  tx buffer value		 						| unsigned short 
						bMSBF				 |  send/receive the words in LSBF/MSBF format	| =true for MSBF = false for LSBF
	Return value	:   received 16 bit data
**************************************************************************************************************************/
unsigned short SPI_Send_Receive_16bit(unsigned short uiTxBuff, bool bMSBF)
{
	//Local variables
	unsigned short uiTemp=0;
	
	//Flush the DMA FIFO
	*pSPIDMAC=0x00000080;
	
	//Flush the SPI FIFO
	*pSPICTL=TXFLSH|RXFLSH;
	
	//Initialize DMA parameters and SPICTL to receive one 16 bit word and send the contents of TXSPI simulataneously
	*pIISPI=(unsigned int)&uiTemp;
	*pIMSPI=1;
	*pCSPI=1;
	
	if(bMSBF==true)
		*pSPICTL=SPIEN|SPIMS|WL16|SENDZ|CLKPL|CPHASE|TIMOD2|MSBF;
	else
		*pSPICTL=SPIEN|SPIMS|WL16|SENDZ|CLKPL|CPHASE|TIMOD2;
		
	*pTXSPI=uiTxBuff;
	*pSPIDMAC=0x00000003;
	
	
	//Wait till DMA is active
	while((*pSPIDMAC&SPIDMAS)!=0);	
	
	return uiTemp;
}

/**************************************************************************************************************************
	Function name	:   SPI_Send_Receive_8bit;
    Description		:   This function sends and receives one 8 bit word over SPI in full duplex mode
	Arguments		:	Parameter			 |	Description		 							| Valid values
						uiTxBuff			 |  tx buffer value		 						| unsigned char
						bMSBF				 |  send/receive the words in LSBF/MSBF format	| =true for MSBF = false for LSBF
	Return value	:   received 8 bit data
**************************************************************************************************************************/
unsigned char SPI_Send_Receive_8bit(unsigned char uiTxBuff, bool bMSBF)
{
	//Local variables
	unsigned char uiTemp=0;
	
	//Flush the DMA FIFO
	*pSPIDMAC=0x00000080;
	
	//Flush the SPI FIFO
	*pSPICTL=TXFLSH|RXFLSH;
	
	//Initialize DMA parameters and SPICTL to receive one 8 bit word and send the contents of TXSPI simulataneously
	*pIISPI=(unsigned int)&uiTemp;
	*pIMSPI=1;
	*pCSPI=1;
	
	if(bMSBF==true)
		*pSPICTL=SPIEN|SPIMS|WL8|SENDZ|CLKPL|CPHASE|TIMOD2|MSBF;
	else
		*pSPICTL=SPIEN|SPIMS|WL8|SENDZ|CLKPL|CPHASE|TIMOD2;
		
	*pTXSPI=uiTxBuff;

	*pSPIDMAC=0x00000003;
	
	//Wait till DMA is active
	while((*pSPIDMAC&SPIDMAS)!=0);	
	
	return uiTemp;
		
}

/**************************************************************************************************************************
	Function name	:   SPI_Send_32bit
    Description		:   This function sends one 32 bit word over SPI 
	Arguments		:	Parameter			 |	Description		 					| Valid values
						uiTxBuff			 |  tx buffer value		 				| unsigned int 32 bit
						bMSBF				 |  send the word in LSBF/MSBF format	| =true for MSBF = false for LSBF
	Return value	:   None
**************************************************************************************************************************/
void SPI_Send_32bit(unsigned int uiTxBuff, bool bMSBF)
{
	//Flush the SPI FIFO
	*pSPICTL=TXFLSH|RXFLSH;
	
	//Send a single 32 bit word using core mode
	if(bMSBF==true)
		*pSPICTL=SPIEN|SPIMS|WL32|SENDZ|CLKPL|CPHASE|TIMOD1|MSBF;
	else
		*pSPICTL=SPIEN|SPIMS|WL32|SENDZ|CLKPL|CPHASE|TIMOD1;
	
	*pTXSPI=(unsigned int)uiTxBuff;
	
	//Wait till buffer is emptied and transfer is complete
	while((*pSPISTAT&TXS)!=0);	
	while((*pSPISTAT&SPIF)==0);
	
	//Disable SPI and flush the FIFO 
	*pSPICTL=TXFLSH|RXFLSH;
}


/**************************************************************************************************************************
	Function name	:   SPI_Send_8bit
    Description		:   This function sends one 8 bit word over SPI 
	Arguments		:	Parameter			 |	Description		 					| Valid values
						uiTxBuff			 |  tx buffer value		 				| unsigned char
						bMSBF				 |  send the word in LSBF/MSBF format	| =true for MSBF = false for LSBF
	Return value	:   None
**************************************************************************************************************************/
void SPI_Send_8bit(unsigned char uiTxBuff, bool bMSBF)
{

	//Flush the SPI FIFO	
	*pSPICTL=TXFLSH|RXFLSH;
	
	//Send a single 8 bit word using core mode
	if(bMSBF==true)
		*pSPICTL=SPIEN|SPIMS|WL8|SENDZ|CLKPL|CPHASE|TIMOD1|MSBF;
	else
		*pSPICTL=SPIEN|SPIMS|WL8|SENDZ|CLKPL|CPHASE|TIMOD1;
	
	*pTXSPI=(unsigned int)uiTxBuff;
	
	//Wait till buffer is emptied and transfer is complete
	while((*pSPISTAT&TXS)!=0);
	while((*pSPISTAT&SPIF)==0);
	
	//Disable SPI and flush the FIFO 
	*pSPICTL=TXFLSH|RXFLSH;
}


/**************************************************************************************************************************
	Function name	:   SPI_Send_Single_Receive_Multiple_32bit
    Description		:   This function sends one 32 bit data (only first word)
    					and receives one or more 32 bit data     
	Arguments		:	Parameter			 |	Description		 							| Valid values
						uiRxBuff 			 |  Rx buffer address							| unsinged int 32 bit	
						uiCount				 |  Total words to be received 					| unsigned int 32 bit
						bMSBF				 |  send/receive the words in LSBF/MSBF format	| =true for MSBF = false for LSBF
	Return value	:   received 32 bit data
**************************************************************************************************************************/
void SPI_Receive_Multiple_32bit(unsigned int* uiRxBuff, unsigned int uiCount, bool bMSBF)
{
	//Flush the DMA FIFO
	*pSPIDMAC=0x00000080;
	
	//Flush the SPI FIFO
	*pSPICTL=TXFLSH|RXFLSH;
	
	//Initialize DMA parameters and SPICTL to receive multiple 32 bit words 
	*pIISPI=(unsigned int)uiRxBuff;
	*pIMSPI=1;
	*pCSPI=uiCount;
	
	if(bMSBF==true)
		*pSPICTL=SPIEN|SPIMS|WL32|SENDZ|CLKPL|CPHASE|TIMOD2|MSBF;
	else
		*pSPICTL=SPIEN|SPIMS|WL32|SENDZ|CLKPL|CPHASE|TIMOD2;

	*pSPIDMAC=0x00000003;
	
	//Wait till DMA is active
	while((*pSPIDMAC&SPIDMAS)!=0);	
	
}

/**************************************************************************************************************************
	Function name	:   SPI_Receive_Multiple_8bit
    Description		:   This function receives multiple 8 bit data     
	Arguments		:	Parameter			 |	Description		 							| Valid values
						uiRxBuff 			 |  Rx buffer address							| unsinged int 32 bit	
						uiCount				 |  Total words to be received 					| unsigned int 32 bit
						bMSBF				 |  send/receive the words in LSBF/MSBF format	| =true for MSBF = false for LSBF
	Return value	:   received 32 bit data
**************************************************************************************************************************/
void SPI_Receive_Multiple_8bit(unsigned char* uiRxBuff, unsigned int uiCount, bool bMSBF)
{
	//Flush the DMA FIFO
	*pSPIDMAC=0x00000080;
	
	//Flush the SPI FIFO
	*pSPICTL=TXFLSH|RXFLSH;
	
	//Initialize DMA parameters and SPICTL to receive multiple 8 bit words 
	*pIISPI=(unsigned int)uiRxBuff;
	*pIMSPI=1;
	*pCSPI=uiCount;
	
	if(bMSBF==true)
		*pSPICTL=SPIEN|SPIMS|WL8|SENDZ|CLKPL|CPHASE|TIMOD2|MSBF;
	else
		*pSPICTL=SPIEN|SPIMS|WL8|SENDZ|CLKPL|CPHASE|TIMOD2;
		
	*pSPIDMAC=0x00000003;
	
	//Wait till DMA is active
	while((*pSPIDMAC&SPIDMAS)!=0);	
	
}

/**************************************************************************************************************************
	Function name	:   SPI_Send_Multiple_8bit
    Description		:   This function sends multiple 8 bit data     
	Arguments		:	Parameter			 |	Description		 							| Valid values
						uiTxBuff 			 |  Tx buffer address							| unsinged int 32 bit	
						uiCount				 |  Total words to be received 					| unsigned int 32 bit
						bMSBF				 |  send/receive the words in LSBF/MSBF format	| =true for MSBF = false for LSBF
	Return value	:   received 32 bit data
**************************************************************************************************************************/
void SPI_Send_Multiple_8bit(unsigned char* uiTxBuff, unsigned int uiCount, bool bMSBF)
{
	//Flush the DMA FIFO
	*pSPIDMAC=0x00000080;
	
	//Flush the SPI FIFO
	*pSPICTL=TXFLSH|RXFLSH;
	
	//Initialize DMA parameters and SPICTL to send multiple 8 bit words 
	*pIISPI=(unsigned int)uiTxBuff;
	*pIMSPI=1;
	*pCSPI=uiCount;
	
	if(bMSBF==true)
		*pSPICTL=SPIEN|SPIMS|WL8|SENDZ|CLKPL|CPHASE|TIMOD2|MSBF;
	else
		*pSPICTL=SPIEN|SPIMS|WL8|SENDZ|CLKPL|CPHASE|TIMOD2;
		
	*pSPIDMAC=0x00000001;
	
	//Wait till DMA is active and external transfer is complete
	while((*pSPIDMAC&SPIDMAS)!=0);		
	while((*pSPISTAT&TXS)!=0);
	Wait_Till_External_Transfer_Complete();
}

/**************************************************************************************************************************
	Function name	:   SPI_Set_Baud_Rate
    Description		:   This function sets the baud rate of the SPI module to the required value
	Arguments		:	Parameter			 |	Description		 						| Valid values
						fBaud				 |  Baud rate (SPICLK speed) required 	    | float 
						fSCLK				 |  SCLK/PCLK speed  						| float
	Return value	:   None
**************************************************************************************************************************/
void SPI_Set_Baud_Rate(float fBaud, float fSCLK)
{
	//Temporary variables
	unsigned int iTemp, iTemp1;	
	
	//Calculate the BAUDR field value
	iTemp = (ceil)(fSCLK/(4*fBaud));	
	
	//Initialize the SPIBAUD regsiter now
	*pSPIBAUD = iTemp << 1;			
}


/**************************************************************************************************************************
	Function name	:   SPI_Set_Baud_Rate
    Description		:   This function sets the baud rate of the SPI module to the required value
	Arguments		:	Parameter			 |	Description		 						| Valid values
						fBaud				 |  Baud rate (SPICLK speed) required 	    | float 
						fSCLK				 |  SCLK/PCLK speed  						| float
	Return value	:   None
**************************************************************************************************************************/
void SPI_Set_Slave_Select(bool bHigh)
{
	if(bHigh==true)
	{
		//Drive chip select high
		SET_SS_HIGH(); 
	}
	else
	{
		//Drive chip select low
		SET_SS_LOW();	
		
	}
		
}

/**************************************** END OF FILE*********************************************************************/
